package ssw.mj.test;

import ssw.mj.Token;

/**
 * Test cases for <code>Token</code> class.
 * 
 * @author AW
 */
public class TokenTest extends IdentifiedTestCase {

	/** Tests various comparisons for equality. */
	public void testEquals () {
		Token t1 = new Token(Token.if_, 12, 25);
		Token t2, t3;
		// equals has to be commutative 
		// so always check both ways, i.e. t1.equals(t2) AND t2.equals(t1)

		/*----- same object -----*/
		assertSame("Same object", t1, t1);
		assertEquals("Same object", t1, t1);

		/*----- two different, but equal objects -----*/
		t2 = new Token(Token.if_, 12, 25);
		assertEquals("Same values (1)", t1, t2);
		assertEquals("Same values (2)", t2, t1);

		t1 = new Token(Token.ident, 20, 222, 0, null);
		t2 = new Token(Token.ident, 20, 222, 0, null);
		assertEquals("Two null strings (1)", t1, t2);
		assertEquals("Two null strings (2)", t2, t1);

		t2 = new Token(Token.ident, 20, 222, 0, "");
		assertEquals("One null, one empty string", t1, t2);
		assertEquals("One empty, one null string", t2, t1);

		t1 = new Token(Token.ident, 20, 222, 0, "");
		assertEquals("Two emtpy strings (1)", t1, t2);
		assertEquals("Two emtpy strings (2)", t2, t1);

		/*---- two different, not equal objects -----*/
		t1 = new Token(Token.if_, 12, 25);
		t2 = new Token(Token.else_, 12, 25);
		assertFalse("Different kind (1)", t1.equals(t2));
		assertFalse("Different kind (2)", t2.equals(t1));

		t2 = new Token(Token.if_, 14, 25);
		assertFalse("Different line (1)", t1.equals(t2));
		assertFalse("Different line (2)", t2.equals(t1));

		t2 = new Token(Token.if_, 12, 2);
		assertFalse("Different column (1)", t1.equals(t2));
		assertFalse("Different column (2)", t2.equals(t1));

		t1 = new Token(Token.charConst, 12, 25, 92, null);
		t2 = new Token(Token.charConst, 12, 25, 95, null);
		assertFalse("Different value (1)", t1.equals(t2));
		assertFalse("Different value (2)", t2.equals(t1));

		t1 = new Token(Token.ident, 12, 25, 0, "blabla");
		t2 = new Token(Token.ident, 12, 25, 0, "bla");
		assertFalse("Different string (1)", t1.equals(t2));
		assertFalse("Different string (2)", t2.equals(t1));
	}
	
	/** Tests string representations for all types of tokens. */
	public void testToString () {
		Token t;
		t = new Token(Token.none, 0, 1);
		assertEquals("none token", "line 0, col 1: none", t.toString());
		t = new Token(Token.ident, 1, 1, 0, "aName");
		assertEquals("ident token", "line 1, col 1: identifier = aName", t.toString());
		t = new Token(Token.number, 2, 1, 1234, "1234");
		assertEquals("number token", "line 2, col 1: number = 1234 (\"1234\")", t.toString());
		t = new Token(Token.charConst, 3, 1, 36, null);
		assertEquals("charConst token", "line 3, col 1: character constant = '$'", t.toString());
		t = new Token(Token.plus, 4, 1);
		assertEquals("plus token", "line 4, col 1: +", t.toString());
		t = new Token(Token.minus, 5, 1);
		assertEquals("minus token", "line 5, col 1: -", t.toString());
		t = new Token(Token.times, 6, 1);
		assertEquals("times token", "line 6, col 1: *", t.toString());
		t = new Token(Token.slash, 7, 1);
		assertEquals("slash token", "line 7, col 1: /", t.toString());
		t = new Token(Token.rem, 8, 1);
		assertEquals("rem token", "line 8, col 1: %", t.toString());
		t = new Token(Token.eql, 9, 1);
		assertEquals("eql token", "line 9, col 1: ==", t.toString());
		t = new Token(Token.neq, 10, 1);
		assertEquals("neq token", "line 10, col 1: !=", t.toString());
		t = new Token(Token.gtr, 11, 1);
		assertEquals("gtr token", "line 11, col 1: >", t.toString());
		t = new Token(Token.geq, 12, 1);
		assertEquals("geq token", "line 12, col 1: >=", t.toString());
		t = new Token(Token.less, 13, 1);
		assertEquals("less token", "line 13, col 1: <", t.toString());
		t = new Token(Token.leq, 14, 1);
		assertEquals("leq token", "line 14, col 1: <=", t.toString());
		t = new Token(Token.and, 15, 1);
		assertEquals("and token", "line 15, col 1: &&", t.toString());
		t = new Token(Token.or, 16, 1);
		assertEquals("or token", "line 16, col 1: ||", t.toString());
		t = new Token(Token.assign, 17, 1);
		assertEquals("assign token", "line 17, col 1: =", t.toString());
		t = new Token(Token.pplus, 18, 1);
		assertEquals("pplus token", "line 18, col 1: ++", t.toString());
		t = new Token(Token.mminus, 19, 1);
		assertEquals("mminus token", "line 19, col 1: --", t.toString());
		t = new Token(Token.semicolon, 20, 1);
		assertEquals("semicolon token", "line 20, col 1: ;", t.toString());
		t = new Token(Token.comma, 21, 1);
		assertEquals("comma token", "line 21, col 1: ,", t.toString());
		t = new Token(Token.period, 22, 1);
		assertEquals("period token", "line 22, col 1: .", t.toString());
		t = new Token(Token.lpar, 23, 1);
		assertEquals("lpar token", "line 23, col 1: (", t.toString());
		t = new Token(Token.rpar, 24, 1);
		assertEquals("rpar token", "line 24, col 1: )", t.toString());
		t = new Token(Token.lbrack, 25, 1);
		assertEquals("lbrack token", "line 25, col 1: [", t.toString());
		t = new Token(Token.rbrack, 26, 1);
		assertEquals("rbrack token", "line 26, col 1: ]", t.toString());
		t = new Token(Token.lbrace, 27, 1);
		assertEquals("lbrace token", "line 27, col 1: {", t.toString());
		t = new Token(Token.rbrace, 28, 1);
		assertEquals("rbrace token", "line 28, col 1: }", t.toString());
		t = new Token(Token.break_, 29, 1);
		assertEquals("break token", "line 29, col 1: break", t.toString());
		t = new Token(Token.class_, 30, 1);
		assertEquals("class_ token", "line 30, col 1: class", t.toString());
		t = new Token(Token.do_, 31, 1);
		assertEquals("do_ token", "line 31, col 1: do", t.toString());
		t = new Token(Token.else_, 32, 1);
		assertEquals("else_ token", "line 32, col 1: else", t.toString());
		t = new Token(Token.final_, 33, 1);
		assertEquals("final_ token", "line 33, col 1: final", t.toString());
		t = new Token(Token.if_, 34, 1);
		assertEquals("if_ token", "line 34, col 1: if", t.toString());
		t = new Token(Token.new_, 35, 1);
		assertEquals("new_ token", "line 35, col 1: new", t.toString());
		t = new Token(Token.print, 36, 1);
		assertEquals("print token", "line 36, col 1: print", t.toString());
		t = new Token(Token.program, 37, 1);
		assertEquals("program token", "line 37, col 1: program", t.toString());
		t = new Token(Token.read, 38, 1);
		assertEquals("read token", "line 38, col 1: read", t.toString());
		t = new Token(Token.return_, 39, 1);
		assertEquals("return_ token", "line 39, col 1: return", t.toString());
		t = new Token(Token.void_, 40, 1);
		assertEquals("void_ token", "line 40, col 1: void", t.toString());
		t = new Token(Token.while_, 41, 1);
		assertEquals("while_ token", "line 41, col 1: while", t.toString());
		t = new Token(Token.eof, 42, 1);
		assertEquals("eof token", "line 42, col 1: end of file", t.toString());
	}
	
	/** Starts a text-mode test runner that runs all token tests. */
	public static void main (String[] args) {
		junit.textui.TestRunner.run(TokenTest.class);	
	}
}
